using System;
using System.Collections.Generic;
using System.Text;
using System.Net.Mail;
using System.Net;
using System.Text.RegularExpressions;

namespace Utils.Misc
{
    /// <summary>
    /// Gmail mail manager
    /// </summary>
    public class Mailer
    {
        #region Variables
        /// <summary>
        /// SMTP login
        /// </summary>
        protected string vAddress;
        /// <summary>
        /// SMTP password
        /// </summary>
        protected string vPassword;
        /// <summary>
        /// Last error message of a sended mail
        /// </summary>
        protected string vSendingErrorMessage = "";
        /// <summary>
        /// SMTP communication port
        /// </summary>
        protected int vPort = 25;
        /// <summary>
        /// SMTP address
        /// </summary>
        protected string vSMTPServer;
        #endregion

        #region Constructeur
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="_Address">SMTP login</param>
        /// <param name="_Password">SMTP password</param>
        /// <param name="_Port">Communication port for the SMTP server</param>
        /// <param name="_SMTPServer">SMTP server's address</param>
        public Mailer(string _Address, string _Password, int _Port, string _SMTPServer)
        {
            vPassword = _Password;
            vAddress = _Address;
        }
        #endregion

        #region Mthodes
        /// <summary>
        /// Send a mail
        /// </summary>
        /// <param name="to">Destination address</param>
        /// <param name="subject">Mail subject</param>
        /// <param name="message">Mail message</param>
        /// <returns>True si la mail est envoy sinon false</returns>
        public bool SendMail(string to, string subject, string message)
        {
            try
            {
                //SMTPClient
                SmtpClient smtp = new SmtpClient();
                smtp.Credentials = new NetworkCredential(vAddress, vPassword);
                smtp.Port = vPort; // 25;//587;
                smtp.Host = vSMTPServer; // "smtp.skynet.be";//"smtp.gmail.com";
                smtp.EnableSsl = false;//true;

                MailMessage mail = new MailMessage();
                mail.From = new MailAddress(vAddress, subject, Encoding.UTF8);
                mail.To.Add(to);
                mail.Subject = "Error Reporting";
                mail.Body = message;
                mail.DeliveryNotificationOptions = DeliveryNotificationOptions.OnFailure;
                mail.ReplyTo = new MailAddress(vAddress);

                smtp.Send(mail);
            }
            catch (Exception ex)
            {
                vSendingErrorMessage = ex.Message;
                return false;
            }
            return true;
        }
        /// <summary>
        /// Indicates whether the mail is valid or not
        /// </summary>
        /// <param name="Email">The email to check</param>
        /// <returns></returns>
        public static bool IsValid(string Email)
        {
            if (Email == null)
            {
                return false;
            }

            int nFirstAT = Email.IndexOf('@');
            int nLastAT = Email.LastIndexOf('@');

            if ((nFirstAT > 0) && (nLastAT == nFirstAT) &&
            (nFirstAT < (Email.Length - 1)))
            {
                // address is ok regarding the single @ sign
                return (Regex.IsMatch(Email, @"(\w+)@(\w+)\.(\w+)"));
            }
            else
            {
                return false;
            }
        }
        #endregion

        #region Accesseurs
        /// <summary>
        /// Gets or sets the login for the SMTP server
        /// </summary>
        public string Address
        {
            get { return vAddress; }
            set { vAddress = value; }
        }
        /// <summary>
        /// Gets or sets the password for the SMTP server
        /// </summary>
        public string Password
        {
            get { return vPassword; }
            set { vPassword = value; }
        }
        /// <summary>
        /// Last error message of a sended mail
        /// </summary>
        public string SendingError { get { return vSendingErrorMessage; } }
        /// <summary>
        /// Gets or sets the communication port of the SMTP server
        /// </summary>
        public int Port
        {
            get { return vPort; }
            set { vPort = value; }
        }
        /// <summary>
        /// Gets or sets the address of the SMTP server
        /// </summary>
        public string SMTPServer
        {
            get { return vSMTPServer; }
            set { vSMTPServer = value; }
        }
        #endregion
    }
}
